package com.sun.org.apache.xerces.internal.impl.xs;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class XSGrammarBucket
{
    Map<String, SchemaGrammar> fGrammarRegistry;
    SchemaGrammar fNoNSGrammar;
    
    public XSGrammarBucket() {
        this.fGrammarRegistry = new HashMap();
        this.fNoNSGrammar = null;
    }
    
    public SchemaGrammar getGrammar(String namespace) {
        if (namespace == null) {
            return this.fNoNSGrammar;
        }
        return (SchemaGrammar)this.fGrammarRegistry.get(namespace);
    }
    
    public void putGrammar(SchemaGrammar grammar) {
        if (grammar.getTargetNamespace() == null) {
            this.fNoNSGrammar = grammar;
        }
        else {
            this.fGrammarRegistry.put(grammar.getTargetNamespace(), grammar);
        }
    }
    
    public boolean putGrammar(SchemaGrammar grammar, boolean deep) {
        SchemaGrammar sg = this.getGrammar(grammar.fTargetNamespace);
        if (sg != null) {
            return sg == grammar;
        }
        if (!deep) {
            this.putGrammar(grammar);
            return true;
        }
        ArrayList<SchemaGrammar> currGrammars = (ArrayList)grammar.getImportedGrammars();
        if (currGrammars == null) {
            this.putGrammar(grammar);
            return true;
        }
        List<SchemaGrammar> grammars = (ArrayList)currGrammars.clone();
        for (int i = 0; i < grammars.size(); ++i) {
            SchemaGrammar sg2 = (SchemaGrammar)grammars.get(i);
            SchemaGrammar sg3 = this.getGrammar(sg2.fTargetNamespace);
            if (sg3 == null) {
                List<SchemaGrammar> gs = sg2.getImportedGrammars();
                if (gs != null) {
                    for (int j = gs.size() - 1; j >= 0; --j) {
                        sg3 = (SchemaGrammar)gs.get(j);
                        if (!grammars.contains(sg3)) {
                            grammars.add(sg3);
                        }
                    }
                }
            }
            else if (sg3 != sg2) {
                return false;
            }
        }
        this.putGrammar(grammar);
        for (int i = grammars.size() - 1; i >= 0; --i) {
            this.putGrammar((SchemaGrammar)grammars.get(i));
        }
        return true;
    }
    
    public boolean putGrammar(SchemaGrammar grammar, boolean deep, boolean ignoreConflict) {
        if (!ignoreConflict) {
            return this.putGrammar(grammar, deep);
        }
        SchemaGrammar sg = this.getGrammar(grammar.fTargetNamespace);
        if (sg == null) {
            this.putGrammar(grammar);
        }
        if (!deep) {
            return true;
        }
        ArrayList<SchemaGrammar> currGrammars = (ArrayList)grammar.getImportedGrammars();
        if (currGrammars == null) {
            return true;
        }
        List<SchemaGrammar> grammars = (ArrayList)currGrammars.clone();
        for (int i = 0; i < grammars.size(); ++i) {
            SchemaGrammar sg2 = (SchemaGrammar)grammars.get(i);
            SchemaGrammar sg3 = this.getGrammar(sg2.fTargetNamespace);
            if (sg3 == null) {
                List<SchemaGrammar> gs = sg2.getImportedGrammars();
                if (gs != null) {
                    for (int j = gs.size() - 1; j >= 0; --j) {
                        sg3 = (SchemaGrammar)gs.get(j);
                        if (!grammars.contains(sg3)) {
                            grammars.add(sg3);
                        }
                    }
                }
            }
            else {
                grammars.remove(sg2);
            }
        }
        for (int i = grammars.size() - 1; i >= 0; --i) {
            this.putGrammar((SchemaGrammar)grammars.get(i));
        }
        return true;
    }
    
    public SchemaGrammar[] getGrammars() {
        int count = this.fGrammarRegistry.size() + ((this.fNoNSGrammar == null) ? 0 : 1);
        SchemaGrammar[] grammars = new SchemaGrammar[count];
        int i = 0;
        for (Map.Entry<String, SchemaGrammar> entry : this.fGrammarRegistry.entrySet()) {
            grammars[i++] = (SchemaGrammar)entry.getValue();
        }
        if (this.fNoNSGrammar != null) {
            grammars[count - 1] = this.fNoNSGrammar;
            return grammars;
        }
        return grammars;
    }
    
    public void reset() {
        this.fNoNSGrammar = null;
        this.fGrammarRegistry.clear();
    }
}
